@@ -9,10 +9,17 @@ Released under MIT license.
 """
 from operator import itemgetter
 
-from tw.api import Widget
+try: #pragma: no cover
+    import tw2.forms
+    from tw2.core import Widget
+    from tw2.core.widgets import WidgetMeta
+except ImportError: #pragma: no cover
+    from tw.api import Widget
+    class WidgetMeta(object):
+        pass
 
-from sprox.viewbase import ViewBase
 from sprox.widgets import SproxDataGrid
+from sprox.viewbase import ViewBase
 from sprox.metadata import FieldsMetadata
 
 class TableBase(ViewBase):
@@ -35,7 +42,7 @@ class TableBase(ViewBase):
       +-----------------------------------+--------------------------------------------+------------------------------+
       | __xml_fields__                    | fields whos values should show as html     |                              |
       +-----------------------------------+--------------------------------------------+------------------------------+
-  
+
 
     see modifiers in :mod:`sprox.viewbase`
 
@@ -48,8 +55,7 @@ class TableBase(ViewBase):
     >>> town_table = TownTable(session)
     >>> print town_table()
     <div>
-    <table
-           id="None" class="grid">
+    <table class="grid">
         <thead>
             <tr>
                     <th  class="col_0">actions</th>
@@ -57,6 +63,7 @@ class TableBase(ViewBase):
                     <th  class="col_2">name</th>
             </tr>
         </thead>
+    <BLANKLINE>
         <tbody>
         </tbody>
     </table>
@@ -71,7 +78,7 @@ class TableBase(ViewBase):
     ...     __model__ = Town
     >>> town_filler = TownFiller(session)
     >>> value = town_filler.get_value()
-    >>> print town_table.__widget__(value=value) #doctest: +SKIP
+    >>> print town_table(value=value) #doctest: +SKIP
     <div xmlns="http://www.w3.org/1999/xhtml">
     <div>
     <table
@@ -126,52 +133,53 @@ class TableBase(ViewBase):
     ...     __model__ = Town
     ...     __omit_fields__ = ['__actions__']
     >>> town_table = TownTable(session)
-    >>> print town_table.__widget__(value=value)
+    >>> print town_table(value=value)
     <div>
-    <table
-           id="None" class="grid">
+    <table class="grid">
         <thead>
             <tr>
                     <th  class="col_0">town_id</th>
                     <th  class="col_1">name</th>
             </tr>
         </thead>
+    <BLANKLINE>
         <tbody>
-            <tr class="even">
-                <td class="col_0">
-                        1
-                </td>
-                <td class="col_1">
-                        Arvada
-                </td>
-            </tr>
-            <tr class="odd">
-                <td class="col_0">
-                        2
-                </td>
-                <td class="col_1">
-                        Denver
-                </td>
-            </tr>
-            <tr class="even">
-                <td class="col_0">
-                        3
-                </td>
-                <td class="col_1">
-                        Golden
-                </td>
-            </tr>
-            <tr class="odd">
-                <td class="col_0">
-                        4
-                </td>
-                <td class="col_1">
-                        Boulder
-                </td>
-            </tr>
+                <tr class="even">
+                    <td class="col_0">
+                            1
+                    </td>
+                    <td class="col_1">
+                            Arvada
+                    </td>
+                </tr>
+                <tr class="odd">
+                    <td class="col_0">
+                            2
+                    </td>
+                    <td class="col_1">
+                            Denver
+                    </td>
+                </tr>
+                <tr class="even">
+                    <td class="col_0">
+                            3
+                    </td>
+                    <td class="col_1">
+                            Golden
+                    </td>
+                </tr>
+                <tr class="odd">
+                    <td class="col_0">
+                            4
+                    </td>
+                    <td class="col_1">
+                            Boulder
+                    </td>
+                </tr>
         </tbody>
     </table>
     </div>
+
     """
 
     #object overrides
@@ -249,15 +257,15 @@ class TableBase(ViewBase):
         field_widgets = []
         for field in self.__fields__:
             widget = field_widget_dict.get(field, None)
-            if widget is None or widget.__class__ is Widget: # yuck
+            if widget is None or widget.__class__ in (Widget, WidgetMeta): # yuck
                 widget = itemgetter(field)
             field_widgets.append(widget)
         args['fields'] = zip(field_headers, field_widgets)
 
         # And, now back to our regularly-scheduled trunk-derived Sprox.
-
         if '__actions__' not in self.__omit_fields__:
             args['pks'] = self.__provider__.get_primary_fields(self.__entity__)
 
         args['xml_fields'] = self.__xml_fields__
+        args.pop('children', None)
         return args
