@@ -1,9 +1,18 @@
 import inspect
-from tw.api import Widget
-from tw.forms import HiddenField
+from sprox.util import name2label, is_widget, is_widget_class
+
+try: #pragma: no cover
+    from tw2.core import Widget
+    from tw2.core.widgets import WidgetMeta
+    from tw2.forms import HiddenField
+except ImportError: #pragma: no cover
+    from tw.api import Widget
+    from tw.forms import HiddenField
+    class WidgetMeta(object):
+        """TW2 WidgetMetaClass"""
+
 from configbase import ConfigBase, ConfigBaseError
-from sqlalchemy.orm import PropertyLoader
-from sqlalchemy.schema import Column
+
 from widgetselector import WidgetSelector
 
 #sa 0.5 support
@@ -17,7 +26,7 @@ class ClassViewer(object):
     """class wrapper to expose items of a class.  Needed to pass classes to TW as params"""
     def __init__(self, klass):
         self.__name__ = klass.__name__
-
+        
 
 class ViewBaseError(Exception):pass
 
@@ -82,12 +91,12 @@ class ViewBase(ConfigBase):
         for attr in dir(self):
             if not attr.startswith('__'):
                 value = getattr(self, attr)
-                if isinstance(value, Widget):
+                if is_widget(value):
                     if not getattr(value, 'id', None):
                         raise ViewBaseError('Widgets must provide an id argument for use as a field within a ViewBase')
                     self.__add_fields__[attr] = value
                 try:
-                    if issubclass(value, Widget):
+                    if is_widget_class(value):
                         self.__field_widget_types__[attr] = value
                 except TypeError:
                     pass
@@ -97,12 +106,17 @@ class ViewBase(ConfigBase):
         widget = getattr(self, '___widget__', None)
         if not widget:
             self.___widget__ = self.__base_widget_type__(**self.__widget_args__)
-            widget = self.___widget__
         return self.___widget__
 
     #try to act like a widget as much as possible
     def __call__(self, *args, **kw):
-        return self.__widget__.__call__(*args, **kw)
+        return self.display(*args, **kw)
+
+    def display(self, *args, **kw):
+        if 'value' not in kw and args:
+            args = list(args)
+            kw['value'] = args.pop(0)
+        return self.__widget__.display(*args, **kw)
 
     @property
     def __widget_args__(self):
@@ -140,13 +154,21 @@ class ViewBase(ConfigBase):
         if inspect.isclass(field):
             entity = ClassViewer(field)
 
-        args = {'id':field_name, 'identity':self.__entity__.__name__+'_'+field_name, 'entity':entity}
-        if isinstance(entity, Column) and entity.default:
-            if isinstance(entity.default.arg, str) or \
-               isinstance(entity.default.arg, unicode) or \
-               isinstance(entity.default.arg, int) or \
-               isinstance(entity.default.arg, float):
-                    args['default'] = entity.default.arg
+        if hasattr(Widget, 'req'):
+            args.update({'id':'sx_'+field_name, 'key':field_name})
+        else: #pragma: no cover
+            args.update({'id':field_name})
+
+        args.update({'name':field_name,
+                'identity':self.__entity__.__name__+'_'+field_name,
+                'entity':entity, 'provider':self.__provider__,
+                'label':name2label(field_name), 'label_text':name2label(field_name)})
+        field_default_value = self.__provider__.get_field_default(entity)
+        if field_default_value[0]:
+            if hasattr(Widget, 'req'):
+                args['value'] = field_default_value[1]
+            else: #pragma: no cover
+                args['default'] = field_default_value[1]
 
         #enum support works completely differently.
         #if isinstance(entity, Column) and isinstance(entity.type, Enum):
@@ -155,9 +177,9 @@ class ViewBase(ConfigBase):
         if field_name in self.__field_attrs__:
             args['attrs'] = self.__field_attrs__[field_name]
 
-        if isinstance(field, PropertyLoader):
-            args['provider'] = self.__provider__
-            args['nullable'] = self.__provider__.is_nullable(self.__entity__, field_name)
+        provider_widget_args = self.__provider__.get_field_provider_specific_widget_args(self.__entity__, field, field_name)
+        if provider_widget_args:
+            args.update(provider_widget_args)
 
         if field_name in self.__field_widget_args__:
             args.update(self.__field_widget_args__[field_name])
@@ -176,6 +198,9 @@ class ViewBase(ConfigBase):
 
         return fields
 
+    def _do_get_field_wiget_type(self, field_name, field):
+        return self.__field_widget_types__.get(field_name, self.__widget_selector__.select(field))
+
     def _do_get_field_widgets(self, fields):
 
         metadata_keys = self.__metadata__.keys()
@@ -196,20 +221,26 @@ class ViewBase(ConfigBase):
                 continue
             if field_name not in metadata_keys:
                 continue
+
             field = self.__metadata__[field_name]
-
-            if inspect.isclass(field):
-                identifier = ClassViewer(field)
-
-            field_widget_type = self.__field_widget_types__.get(field_name,
-                                                                self.__widget_selector__.select(field))
+            field_widget_type = self._do_get_field_wiget_type(field_name, field)
             field_widget_args = self._do_get_field_widget_args(field_name, field)
 
             if field_name in self._do_get_disabled_fields():
                 # in this case, we display the current field, disabling it, and also add
                 # a hidden field into th emix
                 field_widget_args['disabled'] = True
-                widgets[field_name] = (HiddenField(id=field_name.replace('.','_'), identifier=field_name), field_widget_type(**field_widget_args))
+                field_widget_args['attrs'] = {'disabled':True}
+
+                if hasattr(Widget, 'req'):
+                    hidden_id='disabled_' + field_name.replace('.','_')
+                else: #pragma: no cover
+                    hidden_id=field_name.replace('.','_')
+
+                widgets[field_name] = (HiddenField(id=hidden_id, key=field_name,
+                                                   name=field_name,
+                                                   identifier=field_name),
+                                       field_widget_type(**field_widget_args))
             else:
                 widgets[field_name] = field_widget_type(**field_widget_args)
 
