@@ -1,10 +1,18 @@
 from nose.tools import raises, eq_
-from sprox.validatorselector import ValidatorSelector, SAValidatorSelector
+from sprox.validatorselector import ValidatorSelector
+from sprox.sa.validatorselector import SAValidatorSelector
 from sprox.test.base import *
 from formencode.validators  import *
+
+try:
+    import tw2.forms
+    from tw2.core.validation import DateValidator
+except:
+    pass
+
 from formencode.compound import All
 from sqlalchemy import Column, Integer, String
-from sprox.saormprovider import SAORMProvider
+from sprox.sa.provider import SAORMProvider
 from types import NoneType
 
 session = None
