@@ -6,4 +6,4 @@ class TestSproxCheckbox:
         self.widget = SproxCheckBox()
 
     def test_checkbox_invalid_data(self):
-        self.widget(value = 'asdf')
\ No newline at end of file
+        self.widget.display(value = 'asdf')
\ No newline at end of file
